// Copyright 2017-2023 Azul Systems, Inc.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

package jdk.internal.crac;

import jdk.internal.crac.mirror.Context;
import jdk.internal.crac.mirror.impl.BlockingOrderedContext;

public class Core {
    private static ClaimedFDs claimedFDs;

    /**
     * Called by JDK FD resources
     * @return
     */
    public static ClaimedFDs getClaimedFDs() {
        return claimedFDs;
    }

    /**
     * Called by jdk.crac.Core to publish current ClaimedFDs
     */
    public static void setClaimedFDs(ClaimedFDs fds) {
        claimedFDs = fds;
    }

    /**
     * Priorities are defined in the order of registration to the global context.
     * Checkpoint notification will be processed in the order from the bottom to the top of the list.
     * Restore will be done in reverse order: from the top to the bottom.
     *
     * Resources of the same priority will be handled according the context supplied to the priority.
     *
     * Most resources should use priority NORMAL (the lowest priority).
     */
    public enum Priority {
        FILE_DESCRIPTORS(new BlockingOrderedContext<>()),
        PRE_FILE_DESCRIPTORS(new BlockingOrderedContext<>()),
        CLEANERS(new BlockingOrderedContext<>()),
        REFERENCE_HANDLER(new BlockingOrderedContext<>()),
        SEEDER_HOLDER(new BlockingOrderedContext<>()),
        SECURE_RANDOM(new BlockingOrderedContext<>()),
        NATIVE_PRNG(new BlockingOrderedContext<>()),
        EPOLLSELECTOR(new BlockingOrderedContext<>()),
        NORMAL(new BlockingOrderedContext<>());

        private final Context<JDKResource> context;
        Priority(Context<JDKResource> context) {
            jdk.internal.crac.mirror.Core.getGlobalContext().register(context);
            this.context = context;
        }

        public Context<JDKResource> getContext() {
            return context;
        }
    }
}
